@@ -7,23 +7,35 @@ import logging
 import sys
 import time
 import hashlib
+from dataclasses import dataclass
 from os.path import isdir, isfile, exists, join, basename
-from typing import Any, Callable, List, Dict, Optional
+from typing import Any, Callable, List, Dict, Optional, Tuple
 import shutil
 import yaml
-
 import pandas as pd
 import hail as hl
 import click
 from google.cloud import storage
+from hailtop.batch import Batch
+from hailtop.batch.job import Job
+from sample_metadata import (
+    AnalysisApi,
+    AnalysisType,
+    AnalysisStatus,
+    SequenceApi,
+    SampleApi,
+)
 from joint_calling import _version, get_package_path
 
+
 logger = logging.getLogger('joint-calling')
 logger.setLevel('INFO')
 
 
 DEFAULT_REF = 'GRCh38'
 
+REF_BUCKET = 'gs://cpg-reference/hg38/v1'
+
 DATAPROC_PACKAGES = [
     'joint-calling',
     'click',
@@ -36,27 +48,24 @@ DATAPROC_PACKAGES = [
 ]
 
 DRIVER_IMAGE = 'australia-southeast1-docker.pkg.dev/analysis-runner/images/driver'
+
+AR_REPO = 'australia-southeast1-docker.pkg.dev/cpg-common/images'
 GATK_VERSION = '4.2.0.0'
-GATK_DOCKER = (
-    f'australia-southeast1-docker.pkg.dev/cpg-common/images/gatk:{GATK_VERSION}'
-)
+GATK_IMAGE = f'{AR_REPO}/gatk:{GATK_VERSION}'
 # GnarlyGenotyper is in Beta and crashes with NullPointerException when using the
 # official GATK docker, that's why we're using a separate image for it:
-GNARLY_DOCKER = 'australia-southeast1-docker.pkg.dev/cpg-common/images/gnarly_genotyper:hail_ukbb_300K'
-BCFTOOLS_DOCKER = (
-    'australia-southeast1-docker.pkg.dev/cpg-common/images/bcftools:1.10.2--h4f4756c_2'
-)
+GNARLY_IMAGE = f'{AR_REPO}/gnarly_genotyper:hail_ukbb_300K'
+BCFTOOLS_IMAGE = f'{AR_REPO}/bcftools:1.10.2--h4f4756c_2'
+SM_IMAGE = f'{AR_REPO}/sm-api:2.0.3'
 
-TRUTH_GVCFS = dict(
-    syndip=dict(
-        s='syndip',
-        gvcf='gs://gnomad-public/resources/grch38/syndip/full.38.20180222.vcf.gz',
-    ),
-    na12878=dict(
-        s='na12878',
-        gvcf='gs://gnomad-public/resources/grch38/na12878/HG001_GRCh38_GIAB_highconf_CG-IllFB-IllGATKHC-Ion-10X-SOLID_CHROM1-X_v.3.3.2_highconf_PGandRTGphasetransfer.vcf.gz',
-    ),
+TEL_AND_CENT_HT_PATH = join(
+    REF_BUCKET, 'gnomad/telomeres_and_centromeres/hg38.telomeresAndMergedCentromeres.ht'
 )
+LCR_INTERVALS_HT_PATH = join(REF_BUCKET, 'gnomad/lcr_intervals/LCRFromHengHg38.ht')
+SEG_DUP_INTERVALS_HT_PATH = join(
+    REF_BUCKET, 'gnomad/seg_dup_intervals/GRCh38_segdups.ht'
+)
+CLINVAR_HT_PATH = join(REF_BUCKET, 'gnomad/clinvar/clinvar_20190923.ht')
 
 
 def init_hail(name: str, local_tmp_dir: str = None):
@@ -78,6 +87,193 @@ def init_hail(name: str, local_tmp_dir: str = None):
     return local_tmp_dir
 
 
+@dataclass
+class Analysis:
+    """
+    Represents the analysis DB entry
+    """
+
+    id: str
+    type: AnalysisType
+    status: AnalysisStatus
+    output: Optional[str]
+    sample_ids: List[str]
+
+    @staticmethod
+    def from_db(**kwargs):
+        """
+        Convert from db keys, mainly converting id to id_
+        """
+        analysis_type = kwargs.pop('type', None)
+        status = kwargs.pop('status', None)
+        sample_ids = kwargs['sample_ids']
+        output = kwargs.pop('output', [])
+        return Analysis(
+            id=kwargs.pop('id'),
+            type=AnalysisType(analysis_type),
+            status=AnalysisStatus(status),
+            sample_ids=list(set(sorted(sample_ids))),
+            output=output,
+        )
+
+
+def get_latest_complete_analysis(
+    analysis_project: str,
+) -> Dict[Tuple[str, Tuple], Analysis]:
+    """
+    Returns a dictionary that maps a tuple (analysis type, sample ids) to the
+    lastest complete analysis record (represented by a AnalysisModel object)
+    """
+    aapi = AnalysisApi()
+    latest_by_type_and_sids = dict()
+    for a_type in ['cram', 'gvcf', 'joint-calling']:
+        for a_data in aapi.get_latest_complete_analyses_by_type(
+            project=analysis_project,
+            analysis_type=a_type,
+        ):
+            a: Analysis = Analysis.from_db(**a_data)
+            latest_by_type_and_sids[(a_type, tuple(a.sample_ids))] = a
+    return latest_by_type_and_sids
+
+
+def make_sm_in_progress_job(
+    b: Batch, analyais_type: str, analysis_project: str, analysis_id: str
+) -> Job:
+    """
+    Creates a job that updates the sample metadata server entry analysis status
+    to in-progress
+    """
+    return make_sm_update_status_job(
+        b, analyais_type, 'in-progress', analysis_project, analysis_id
+    )
+
+
+def make_sm_completed_job(
+    b: Batch, analyais_type: str, sm_db_name: str, analysis_id: str
+) -> Job:
+    """
+    Creates a job that updates the sample metadata server entry analysis status
+    to completed
+    """
+    return make_sm_update_status_job(
+        b, analyais_type, 'completed', sm_db_name, analysis_id
+    )
+
+
+def make_sm_update_status_job(
+    b: Batch, analysis_type: str, status: str, sm_db_name: str, analysis_id: str
+) -> Job:
+    """
+    Creates a job that updates the sample metadata server entry analysis status.
+    """
+    assert status in ['in-progress', 'failed', 'completed', 'queued']
+    j = b.new_job(f'SM: update {analysis_type} to {status}')
+    j.image(SM_IMAGE)
+    j.command(
+        f"""
+set -o pipefail
+set -ex
+
+export GOOGLE_APPLICATION_CREDENTIALS=/gsa-key/key.json
+gcloud -q auth activate-service-account --key-file=$GOOGLE_APPLICATION_CREDENTIALS
+export SM_DEV_DB_PROJECT={sm_db_name}
+export SM_ENVIRONMENT=PRODUCTION
+
+cat <<EOT >> update.py
+from sample_metadata.api import AnalysisApi
+from sample_metadata import AnalysisUpdateModel
+aapi = AnalysisApi()
+aapi.update_analysis_status(
+    analysis_id='{analysis_id}',
+    analysis_update_model=AnalysisUpdateModel(status='{status}'),
+)
+EOT
+python update.py
+    """
+    )
+    return j
+
+
+def find_inputs_from_db(project):
+    """
+    Determine inputs from SM DB
+    """
+    aapi = AnalysisApi()
+    seqapi = SequenceApi()
+    sapi = SampleApi()
+
+    # Get samples in latest analysis
+    latest_analysis = aapi.get_latest_complete_analyses_by_type(
+        analysis_type='joint-calling', project=project
+    )
+    latest_analysis_sample_ids = [
+        analysis['sample_ids'] for analysis in latest_analysis
+    ]
+    samples_in_latest_analysis = [
+        sample_id for sublist in latest_analysis_sample_ids for sample_id in sublist
+    ]
+
+    active_samples = sapi.get_samples(
+        body_get_samples_by_criteria_api_v1_sample_post={
+            'project_ids': [project],
+            'active': True,
+        }
+    )
+    active_sample_ids = [active_sample['id'] for active_sample in active_samples]
+
+    new_samples = list(set(active_sample_ids) - set(samples_in_latest_analysis))
+
+    deleted_samples = list(set(samples_in_latest_analysis) - set(active_sample_ids))
+
+    inputs = []
+
+    # Get all sequence metadata for the list of processed samples
+    sequences_data = seqapi.get_sequences_by_sample_ids(request_body=new_samples)
+
+    new_sample_gvcfs = aapi.get_latest_gvcfs_for_samples(new_samples)
+
+    for new_gvcf in new_sample_gvcfs:
+        sample_id = new_gvcf.get('sample_ids')[0]
+        current_seq_data = next(
+            seq_data
+            for seq_data in sequences_data
+            if seq_data['sample_id'] == sample_id
+        )
+        sample_information = {
+            's': sample_id,
+            'population': 'EUR',
+            'gvcf': new_gvcf.get('output'),
+            'r_contamination': current_seq_data.get('meta').get('raw_data.FREEMIX'),
+            'r_chimera': current_seq_data.get('meta').get('raw_data.PCT_CHIMERAS'),
+            'r_duplication': current_seq_data.get('meta').get(
+                'raw_data.PERCENT_DUPLICATION'
+            ),
+            'median_insert_size': current_seq_data.get('meta').get(
+                'raw_data.MEDIAN_INSERT_SIZE'
+            ),
+            'operation': 'add',
+        }
+
+        inputs.append(sample_information)
+
+    for sample in deleted_samples:
+        sample_information = {
+            's': sample,
+            'population': None,
+            'gvcf': None,
+            'r_contamination': None,
+            'r_chimera': None,
+            'r_duplication': None,
+            'median_insert_size': None,
+            'operation': 'delete',
+        }
+        inputs.append(sample_information)
+
+    df = pd.DataFrame(inputs)
+
+    return df
+
+
 def find_inputs(
     input_buckets: List[str],
     input_metadata_buckets: Optional[List[str]] = None,
@@ -181,16 +377,6 @@ def find_inputs(
         else:
             df.loc[matching_sn[0], ['gvcf']] = gp
     df = df[df.gvcf.notnull()]
-
-    # Adding truth samples
-    # df['truth'] = False
-    # for truth_sample in TRUTH_GVCFS.values():
-    #     df.loc[truth_sample['s'], ['s', 'gvcf', 'truth']] = [
-    #         truth_sample['s'],
-    #         truth_sample['gvcf'],
-    #         True,
-    #     ]
-
     return df
 
 
@@ -323,6 +509,7 @@ def get_mt(
     meta_ht: hl.Table = None,
     add_meta: bool = False,
     release_only: bool = False,
+    passing_sites_only: bool = False,
 ) -> hl.MatrixTable:
     """
     Wrapper function to get data with desired filtering and metadata annotations
@@ -338,20 +525,17 @@ def get_mt(
     :param add_meta: whether to add metadata to MT in 'meta' column
     :param release_only: whether to filter the MT to only samples available for
         release (can only be used if metadata is present)
+    :param passing_sites_only: whether to filter the MT to only variants with
+        nothing in the filter field (e.g. passing soft filters)
     :return: MatrixTable with chosen annotations and filters
     """
     mt = hl.read_matrix_table(mt_path)
 
-    # keying by locus and allele
-    mt = hl.MatrixTable(
-        hl.ir.MatrixKeyRowsBy(
-            mt._mir,  # pylint: disable=protected-access
-            ['locus', 'alleles'],
-            # Prevent hail from running sort on genotype MT which is already sorted
-            # by a unique locus
-            is_sorted=True,
-        )
-    )
+    if passing_sites_only:
+        try:
+            mt = mt.filter_rows(mt.filters.length() == 0)
+        except AttributeError:
+            pass
 
     if hard_filtered_samples_to_remove_ht is not None:
         mt = mt.filter_cols(
@@ -376,6 +560,7 @@ def get_mt(
             & hl.any(lambda a: hl.is_indel(mt.alleles[0], a), mt.alleles[1:])
             & hl.any(lambda a: hl.is_snp(mt.alleles[0], a), mt.alleles[1:]),
         )
+        # Will use GT instead of LGT
         mt = hl.experimental.sparse_split_multi(mt, filter_changed_loci=True)
 
     return mt
@@ -427,3 +612,7 @@ def get_filter_cutoffs(
             filter_cutoffs_d = yaml.load(f)
 
     return filter_cutoffs_d
+
+
+if __name__ == '__main__':
+    find_inputs_from_db('tob-wgs')
